// Diffusion.cpp: implementation of the Diffusion class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "soft.h"
#include "Diffusion.h"
#include "Geometry2.h"
#include "math.h"
#include "myError.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Diffusion::Diffusion()
{
   input = NULL;
   nPts =0;
   iterN = 10;
   step = 1;
}


Diffusion::~Diffusion()
{
  if (nPts >0)
	  delete []faceNmls;
}


void Diffusion::smoothNormal(int trId, float *diffusedN)
{
   int curTr, k;
   float newN[3] = {0,0,0};
   float w = 0;
   float totalW =0;
   float *p2 = new float[3];
   float *p1 = new float[3];
   float *n1 = new float[3];
   float *n2 = new float[3];
   float* tempP0 = new float[3];
   float* tempP1 = new float[3];
   float* tempP2 = new float[3];
 
   LList* nei = new LList();
   float d, K;
   
   for (k=0; k<3; k++)
   {
    tempP0[k] = input->Points[input->GetPointId(trId,0)][k];
    tempP1[k] = input->Points[input->GetPointId(trId,1)][k];
    tempP2[k] = input->Points[input->GetPointId(trId,2)][k];
   }

    Geometry::trlOrtho(tempP0, tempP1,  tempP2, p1);
 
	
	n1[0] = faceNmls[trId][0];
	n1[1] = faceNmls[trId][1];
	n1[2] = faceNmls[trId][2];

    Geometry::trNormal(tempP0, tempP1,  tempP2, n1);
    Geometry::normalize(n1);
 
    input->trlCircleNei(trId, nei);
    nei->Head();
    for (int i=0; i<nei->length(); i++)
    {
 	 curTr = nei->Retrieve();

     for (k=0; k<3; k++)
	 {
      tempP0[k] = input->Points[input->GetPointId(curTr,0)][k];
      tempP1[k] = input->Points[input->GetPointId(curTr,1)][k];
      tempP2[k] = input->Points[input->GetPointId(curTr,2)][k];
	 }

	 Geometry::trlOrtho(tempP0, tempP1,  tempP2, p2);
     
	 n2[0] = faceNmls[curTr][0];
	 n2[1] = faceNmls[curTr][1];
	 n2[2] = faceNmls[curTr][2];

	 d = Geometry::dist(p1, p2);

     float Phi = Geometry::dot(n1,n2);

	 if ( Geometry::isZero(Phi+1.0f) ) Phi = -1.0f;
     if ( Geometry::isZero(Phi-1.0f) ) Phi = 1.0f;	 

	 if (Geometry::isZero(d))
             K = acosf(Phi);
	   else
	     K = acosf(Phi)/d;

	 w = input->trsArea[curTr]*(float)exp(-K*K/step);
     totalW += w;
          
     newN[0] += w*n2[0];
     newN[1] += w*n2[1];
     newN[2] += w*n2[2];    
		  
     nei->Forward();	
	}
    
    if (!Geometry::isZero(totalW))
	{
     diffusedN[0] = newN[0]/totalW;
     diffusedN[1] = newN[1]/totalW;
     diffusedN[2] = newN[2]/totalW; 
	}
	else
	{
     diffusedN[0] = newN[0];
     diffusedN[1] = newN[1];
     diffusedN[2] = newN[2]; 
	}

    Geometry::normalize(diffusedN);
   
		 
   delete []p1;
   delete []p2;
   delete []n1;
   delete []n2;
   delete []tempP0;
   delete []tempP1;
   delete []tempP2;
   delete nei;

 }

void Diffusion::Proceed()
{
 	float *v = new float[3];
    float *p1= new float[3];
    float *p2= new float[3];
    float shift[3];
    float* tempP0 = new float[3];
    float* tempP1 = new float[3];
    float* tempP2 = new float[3];
    float* diffusedN = new float[3];
    int ptId;


	float **newPoints;
    newPoints = new float* [nPts];   //
    for ( int i=0; i<nPts; i++ )
	{
	 newPoints[i] = new float[3];
	}

	for (int s=0; s<iterN; s++)
	{

	  for (int trId=0; trId<input->numTrs; trId++)
	  {
         for (int k=0; k<3; k++)
		 {
          tempP0[k] = input->Points[input->GetPointId(trId,0)][k];
          tempP1[k] = input->Points[input->GetPointId(trId,1)][k];
          tempP2[k] = input->Points[input->GetPointId(trId,2)][k];
		 }

		Geometry::trNormal(tempP0, tempP1,  tempP2, v);

		if (s != 0)
		{
         input->trsArea[ptId] = 0.5f*Geometry::Norm(v); 
		}

		Geometry::normalize(v);

		faceNmls[trId][0] = v[0];
        faceNmls[trId][1] = v[1];
        faceNmls[trId][2] = v[2];
	  }

	  for (ptId=0; ptId<input->numPts; ptId++)
	  {

	    shift[0]=shift[1]=shift[2]= 0;

        p2[0] = this->input->Points[ptId][0]; 
        p2[1] = this->input->Points[ptId][1]; 
        p2[2] = this->input->Points[ptId][2]; 

        for (int j=0; j<input->Neis[ptId].numNei; j++)
		{	     
	     int incId = input->Neis[ptId].pointNei[j];

		 for (int k=0; k<3; k++)
		 {
		  tempP0[k] = input->Points[input->GetPointId(incId,0)][k];
          tempP1[k] = input->Points[input->GetPointId(incId,1)][k];
          tempP2[k] = input->Points[input->GetPointId(incId,2)][k];
		 }

	     Geometry::trlOrtho(tempP0, tempP1,  tempP2, p1);
  
		 Geometry::vector(p1, p2, v);
		 Geometry::normalize(v);
         smoothNormal(incId, diffusedN);
 
		 float scale = Geometry::dot(v,diffusedN);
		 
		 shift[0] += scale*diffusedN[0]*input->trsArea[incId];
         shift[1] += scale*diffusedN[1]*input->trsArea[incId];
         shift[2] += scale*diffusedN[2]*input->trsArea[incId]; 
		}

	    for (int k=0; k<3; k++)

		{
		  if (Geometry::isZero(input->nbhdArea[ptId]))
             newPoints[ptId][k] = p2[k] + shift[k];
		  else
             newPoints[ptId][k] = p2[k] + shift[k]/input->nbhdArea[ptId];
		}
	  }

	  for (int j=0; j<input->numPts; j++)
	  {
       this->input->setPoint(j, newPoints[j]);
	  }

    for (int ptId=0; ptId<input->numPts; ptId++)
	{

     for (int neiId=0; neiId<input->Neis[ptId].numNei; neiId++)
     {
	  int trId = input->Neis[ptId].pointNei[neiId];
      input->nbhdArea[ptId] += input->trsArea[trId];
	 }
  }


	}     //for all iterations
	
	delete [] v;
	delete [] tempP0;
	delete [] tempP1;
	delete [] tempP2;
	delete []p1;
	delete []p2;
	delete []diffusedN;

    for (i=0; i<nPts; i++ )
	{
	 delete [] newPoints[i];
	}
    delete newPoints;

}

void Diffusion::SetInput(polyData *dataIn)
{  
  this->input = dataIn;
  this->nPts = dataIn->numPts;
  faceNmls = new float[dataIn->numTrs][3];
}

void Diffusion::SetNumberOfIteration(int N)
{
   this->iterN = N;
}

void Diffusion::SetStep(float param)
{
  this->step = param;
}
