// Geometry2.cpp: implementation of the Geometry class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "soft.h"
#include "Geometry2.h"
#include "math.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


static double myPI =  acos(-1.0);

Geometry::Geometry()
{

}

Geometry::~Geometry()
{

}

void Geometry::cross( float *v1, float *v2, float *res)
{
  res[0] = v1[1]*v2[2] - v1[2]*v2[1];
  res[1] = v1[2]*v2[0] - v1[0]*v2[2];
  res[2] = v1[0]*v2[1] - v1[1]*v2[0];
}

float Geometry::dot( float *v1, float *v2)
{
  return (v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2]);
}

int Geometry::isZero(float s)            //to compear abs value with 0
{  
  if (fabs(s) < 1e-6)  return(1);
  else return(0);
}

void Geometry::line_and_plane(float *s, float *lambda, int parallel, 
					float *a, float *ab, float *planeNormal, 
					float *planePoint)
{
// s is intersection point
// lambda is parameter to intersection point
// a and ab is point on line and direction of line
// planeNormal and planePoint are normal to plane and point on plane
   const float INF= (float) 1e20;

   float N_ab =0;
   float N_a =0;
 
   N_ab = dot(planeNormal, ab);
   N_a = dot(planeNormal, a);
   
   float c;
   c = dot(planeNormal,planePoint);
   if (isZero(N_ab))
   {
	 parallel = TRUE;
     if (isZero(N_a-c)) *lambda=0;
     else *lambda=(float)INFINITE;
   }
   else 
   {
	 parallel = FALSE;
	 *lambda = (c - N_a)/N_ab;
   }                           //end if (isZero())/else

   for (int i=0; i<3; i++)
   {
     s[i] = a[i]+ *lambda*ab[i];
   }
}

// to check if edge vertices are on different sides of plane
int Geometry::ifIntersect(float *Qi, float *Qii, float *planeNormal, float *planePoint)
{
  int numQi=0;
  if ( dot(Qi, planeNormal) - dot(planePoint, planeNormal) > 0)
        numQi= 1;
  else numQi= -1;

  int numQii=0;
  if ( dot(Qii, planeNormal) - dot(planePoint, planeNormal) >0 )
      numQii= 1;
  else numQii= -1;
   
  if (numQi*numQii <0) return (1);
  else return 0;

}


float Geometry::Norm(float *v)
{
 return ((float) sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]) );
}

void Geometry::normalize(float *v)
{
  float d =Norm(v);
  if (isZero(d) ) d=1;
  
  v[0] /= d;
  v[1] /= d;
  v[2] /= d;
}

float Geometry::trArea(float *p0, float *p1, float *p2)
{
   float *a1 =  new float[3];
   float *a2 =  new float[3];
   float *c =  new float[3];
           
         //   a1/\a2
		 //   /____\
         // p_1  b  p_2
		
    for(int k=0;k<3;k++)
	 {
	  a1[k] = p1[k]- p0[k];
      a2[k] = p2[k]- p0[k];
	 }
	
	cross(a1,a2,c);                  // c = a1 x a2
   	float area = (float) 0.5f *Norm(c);

	delete []a1;
	delete []a2;
	delete []c;

    return area;
}

void Geometry::trNormal(float *p0, float *p1, float *p2, float *n)
{
   float *a1 =  new float[3];
   float *a2 =  new float[3];
            
         //   a1/\a2
		 //   /____\
         // p_1  b  p_2
		
    for(int k=0;k<3;k++)
	 {
	  a1[k] = p1[k]- p0[k];
      a2[k] = p2[k]- p0[k];
	 }

	cross(a1,a2,n); // n = a1 x a2

	delete []a1;
	delete []a2;
}


inline float Geometry::det(float *row1, float *row2, float *row3)
{
  return( row1[0]*row2[1]*row3[2] + row1[1]*row2[2]*row3[0] +
     row1[2]*row2[0]*row3[1] - row1[2]*row2[1]*row3[0] -
        row1[0]*row2[2]*row3[1] - row1[1]*row2[0]*row3[2]);
}

void Geometry::MatrByVect(float M[16], float v[4], float res[4])
{     
  res[0]=M[ 0]*v[0]+M[ 4]*v[1]+M[ 8]*v[2]+M[12]*v[3];    
  res[1]=M[ 1]*v[0]+M[ 5]*v[1]+M[ 9]*v[2]+M[13]*v[3];    
  res[2]=M[ 2]*v[0]+M[ 6]*v[1]+M[10]*v[2]+M[14]*v[3];    
  res[3]=M[ 3]*v[0]+M[ 7]*v[1]+M[11]*v[2]+M[15]*v[3]; 
}

// vector p2p1
void Geometry::vector(float *p1, float *p2, float *v)              
{
  v[0] = p1[0] - p2[0];
  v[1] = p1[1] - p2[1];
  v[2] = p1[2] - p2[2];
}

// to get median intersection point

void Geometry::trlOrtho(float *p0,  float *p1, float *p2, float *centroid)
{
  centroid[0] = (p0[0]+p1[0]+p2[0])/3.0f;
  centroid[1] = (p0[1]+p1[1]+p2[1])/3.0f;
  centroid[2] = (p0[2]+p1[2]+p2[2])/3.0f;
}

float Geometry::dist(float *p1, float *p2)
{
  float Vx = p1[0] -p2[0];
  float Vy = p1[1] -p2[1];
  float Vz = p1[2] -p2[2];

  return( (float)sqrt(Vx*Vx +Vy*Vy +Vz*Vz) );
}