// Normals.cpp: implementation of the Normals class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "soft.h"
#include "Normals.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Normals::Normals()
{
  this->output= NULL;
  this->numPts = 0;
}

Normals::Normals(int nPts)
{
  this->output = new float* [nPts];   //
  for ( int i=0; i<nPts; i++ )
  {
	this->output[i] = new float[3];
  } 

  this->numPts = nPts;
}


Normals::~Normals()
{
  if (this->output)
  {
   for (int i=0; i<this->numPts; i++ )
	{
	  delete [] this->output[i];
	}
 
   delete [] this->output;
  }

}

float** Normals::getOutput()
{
  return output;
}

void Normals::setInput(polyData *const data)
{
  this->input = data;
}

void Normals::Proceed()
{
   taubinNormals();
}


//normals at verteces (as Taubin)
void Normals::taubinNormals()
{

  float *p_1, *p_2, *p_0;
  int numPts = this->numPts;
  int k, trId;
  int pointId0, pointId1, pointId2;

  for (int s=0; s<numPts; s++)
	 {
      for (int k=0; k<3; k++)     
	  {
        output[s][k] = 0.0;
	  }
	}

    for (trId=0; trId<input->numTrs; trId++)
	{
	  pointId0 = input->GetPointId(trId,0);
	  pointId1 = input->GetPointId(trId,1);
	  pointId2 = input->GetPointId(trId,2);

	  p_0 = input->Points[pointId0];
	  p_1 = input->Points[pointId1];
	  p_2 = input->Points[pointId2];   

	  float a1[3], a2[3], c[3];

         //   a1/\a2
		 //   /____\
         // p_1  b  p_2
		
      for(k=0;k<3;k++)
	  {
	    a1[k] = p_1[k]- p_0[k];
        a2[k] = p_2[k]- p_0[k];
	  }

      Geometry::cross(a1,a2,c);                  // c = a1 x a2
      
	  for (k=0; k<3; k++)             //coordinates
		 {
		  this->output[pointId0][k] += c[k];
		  this->output[pointId1][k] += c[k];
		  this->output[pointId2][k] += c[k];
		 }
	  
	}   // for all trs


// normalize found normals
	for (int ptId=0; ptId<numPts; ptId++)
	{
	  Geometry::normalize(this->output[ptId]);
	}
	
}
	
void Normals::getVertexNormal(int ptId, polyData *data, float *n)
{
   int k;
   float *p_1, *p_2, *p_0;
   int pointId0, pointId1, pointId2;

   
   data->Neis[ptId].pointNei;

   for (int i=0; i< data->Neis[ptId].numNei; i++)
   {
     int trId = data->Neis[ptId].pointNei[i];

  
    pointId0 = data->GetPointId(trId,0);
	pointId1 = data->GetPointId(trId,1);
	pointId2 = data->GetPointId(trId,2);

	p_0 = data->Points[pointId0];
	p_1 = data->Points[pointId1];
	p_2 = data->Points[pointId2];   

	float a1[3], a2[3], c[3];

         //   a1/\a2
		 //   /____\
         // p_1  b  p_2
		
     for(k=0;k<3;k++)
	  {
	    a1[k] = p_1[k]- p_0[k];
        a2[k] = p_2[k]- p_0[k];
	  }

     Geometry::cross(a1,a2,c);                  // c = a1 x a2
      
	 for (k=0; k<3; k++)             //coordinates
		 {
		  n[k] += c[k];
		 }
	  
	}   // for all trs

	  Geometry::normalize(n);
}
