//curvClass.cpp    
//-------------------------------------------------
#include "central.h"
#include "curvClass.h" 


curvature::curvature(){}
 
curvature::curvature(int numPts)
{
    this->Tmax = new float* [numPts];   
	for (int i=0; i<numPts; i++)
	{
	  this->Tmax[i]= new float[3];
 	}


	this->Tmin = new float* [numPts];   
	for ( i=0; i<numPts; i++ )
	{
	  this->Tmin[i] = new float[3];
	}

    this->Kmax = new float[numPts];
	for ( i=0; i<numPts; i++ )
	{
	  this->Kmax[i] = 0.0;
	}

    this->Kmin = new float[numPts];
	for (i=0; i<numPts; i++)
	{
      this->Kmin[i] = 0.0;
	}

}

curvature::~curvature()
{

}

void curvature::getPrincCurv(polyModel *model)
{

  int numPts = model->numPts;
  int numNei = 0;
  int k, s, t;
  

  float scale = 0;
  int *inc = new int[100];
  int *common = new int[10];

  MATRIX3X3 Mvi;

  for (int i=0; i<numPts; i++)                 //for all points
  {
   float Wij;
   float Kij;
   float Tij[3];
   ZeroMatrix(Mvi);

   int numInc = model->incidentVts(i, inc);

   for (int incId=0; incId<numInc; incId++)             //for all neighbors
	{ 
	  float Vij[3];
	  int j = inc[incId];

	  for (k=0; k<3; k++)
	  {
        Vij[k] = model->Points[j][k] - model->Points[i][k];
	  }

      scale = dot(Vij, model->Normals[i]);
	  for (k=0; k<3; k++)
	  {
        Tij[k] =Vij[k] - scale * model->Normals[i][k];
	  }

	  normalize(Tij);
 	  
	  Kij =0;
	  float normVij = norm(Vij);
	  if (isZero(normVij)) normVij=1;
	 
	  Kij = 2*dot(model->Normals[i],Vij)/(normVij*normVij); 
	
	  //generate weights 	       
	  int edgeNei = model->GetEdgeNei(i, j, common);
	
      Wij=0;
		  
	  for (s=0; s<edgeNei; s++)
	  {
        Wij += model->trsArea[common[s]];
	  }
 
	  float totalWij = 2*model->nbhdArea[i]; 
	  Wij /= totalWij;
	  
	  for(int row=0; row<3; row++)
		  for(int column=0; column<3; column++)
			Mvi[row][column] += Wij*Kij*Tij[row]*Tij[column]; 

   }     // for all neighbors
	   
     
      float Wvi[3];
 
	  float E1[3] = {1, 0, 0};

	  float temp1[3], temp2[3];
	  for (k=0; k<3; k++)
	  {
        temp1[k] = E1[k] - model->Normals[i][k];
		temp2[k] = E1[k] + model->Normals[i][k]; 
	  }

      if (norm(temp1) > norm(temp2) )
	  {
	   for (k=0; k<3; k++)
	   {
        Wvi[k] = temp1[k]/norm(temp1);
	   }
	  }

	  else 
	  {
	   for (k=0; k<3; k++)
	   {
        Wvi[k] = temp2[k]/norm(temp2);
	   }
	  }
	 
	  MATRIX3X3 WviWviT;
	  ZeroMatrix(WviWviT);

      for (int row =0; row<3; row++)
		for (int col =0; col<3; col++)
		  {
            WviWviT[row][col] = Wvi[row]*Wvi[col];
		  }

	  MATRIX3X3 Unit;
	  UnitMatrix(Unit);

      MATRIX3X3 Qvi;
	  ZeroMatrix(Qvi);
	  
  	  for (s=0; s<3; s++)
	  {
	   for (t=0; t<3; t++)
	   {
 		Qvi[s][t] = Unit[s][t] - 2*WviWviT[s][t];
	   }
	  }

	  MATRIX3X3 last, tmp, QviT;
	  ZeroMatrix(tmp);
	  ZeroMatrix(last);
	  ZeroMatrix(QviT);

	  TransposeMatrix(QviT, Qvi);

      MultMatrix(tmp, Mvi, Qvi);
	  MultMatrix(last, QviT, tmp);

	  float lambda1, lambda2;

	  float m11 = last[1][1];
	  float m12 = last[2][1];
	  float m22 = last[2][2]; 

      float *tempT1 = new float[3];
	  float *tempT2 = new float[3];
	  
	  tempT1[0] = (1-isZero(Qvi[0][1]))*Qvi[0][1];      //to make tempT1[i] zero
	  tempT1[1] = Qvi[1][1];                           //when Qvi[i][1] is very small
	  tempT1[2] = Qvi[2][1];
      
	  tempT2[0] = Qvi[0][2];
	  tempT2[1] = Qvi[1][2];
	  tempT2[2] = Qvi[2][2];

      float *T1 = new float[3];
	  float *T2 = new float[3];
	  float myCos, mySin;

	  if (isZero(m12))
	  {
		lambda1 = m11;
        lambda2 = m22;
		if (m11=m22) {myCos=mySin=0;}
		else
		{
		  myCos = 1;
		  mySin =0;
		}
	  }
	  else
	  {
	   float Theta = 0.5*(m22-m11)/m12;

	   float f = (float) 1.0/(Abs(Theta)+sqrt(Theta*Theta+1));

	   if (Theta <0)  f = -f;

	   myCos = 1/sqrt(f*f+1);
	   mySin = myCos*f;   

	   lambda1 = m11-f*m12;
	   lambda2 = m22+f*m12;
      // lambda1 = myCos*myCos*m1 + mySin*mySin*m2 - 2*myCos*mySin*m; 
   	  // lambda2 = mySin*mySin*m1 + myCos*myCos*m2 + 2*mySin*myCos*m;
	  } 

	  for (k=0; k<3; k++)
	  {
        T1[k] = myCos*tempT1[k] - mySin*tempT2[k];
		T2[k] = myCos*tempT1[k] + mySin*tempT2[k];
	  }
	
	  float norm1 = norm(T1);
	  float norm2 = norm(T2);;
	  if (norm1 ==0) norm1=1;
	  if (norm2 ==0) norm2=1;


	  float Kp1 = 3*lambda1 - lambda2;  // T1
	  float Kp2 = 3*lambda2 - lambda1;  // T2

      if (Kp1 >= Kp2)              
	  {
        this->Kmax[i] = Kp1;
		this->Kmin[i] = Kp2;	
		for (k=0; k<3; k++)
		{
		 this->Tmax[i][k] = T1[k]/norm1;
		 this->Tmin[i][k] = T2[k]/norm2;
		}
	  }
      else if (Kp1<Kp2)
	  {
        this->Kmax[i] = Kp2;
		this->Kmin[i] = Kp1;
		for (k=0; k<3; k++)
		{
		 this->Tmax[i][k] = T2[k]/norm2;
		 this->Tmin[i][k] = T1[k]/norm1;
		}
	  }
 
}                        //for all points
     delete[] inc;
     delete[] common;

}


float curvature::intrpolate(float *A, int Qi, int Qii, 
				         polyModel *model, float *Kmax_or_min)
{
 float 	AQi =0;
 float t;
 float KA;
 float temp[3];

 temp[0] = model->Points[Qi][0] - A[0];
 temp[1] = model->Points[Qi][1] - A[1];
 temp[2] = model->Points[Qi][2] - A[2];

 AQi = norm(temp);

 temp[0] = model->Points[Qi][0] - model->Points[Qii][0];
 temp[1] = model->Points[Qi][1] - model->Points[Qii][1];
 temp[2] = model->Points[Qi][2] - model->Points[Qii][2];
 
 float QiQii = norm(temp);

 t = AQi/QiQii;

 KA = Kmax_or_min[Qi]*(1-t) + Kmax_or_min[Qii]*t;

 return KA;
   
}

