#include "drawScene.h"
#include "central.h"

#define glRGB(x, y, z)	glColor3ub((GLubyte)x, (GLubyte)y, (GLubyte)z)

extern polyModel *modelPtr;
extern LList *ridgePtr;
extern LList *ridgeEdgesPtr;
extern curvature *modelCurv;

// Define a constant for the value of PI
const float GL_PI=3.1415f;

// Rotation amounts
static GLfloat xRot = 0.0f;
static GLfloat zRot = 0.0f;

 
void SetupRC(void)
{

	// Light values and coordinates
	GLfloat  whiteLight[] = { 0.45f, 0.45f, 0.45f, 1.0f };
	GLfloat  sourceLight[] = { 0.25f, 0.25f, 0.25f, 1.0f };
	GLfloat	 lightPos[] = { 0.0f, 0.0f, 250.0f, 0.0f };

	glEnable(GL_DEPTH_TEST);	// Hidden surface removal
	glFrontFace(GL_CCW);		// Counter clock-wise polygons face out
//	glEnable(GL_CULL_FACE);		// Do not calculate inside 
//	glShadeModel(GL_FLAT);
	

	// Enable lighting
	glEnable(GL_LIGHTING);

	// Setup and enable light 0
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,whiteLight);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	glLightfv(GL_LIGHT0,GL_AMBIENT,sourceLight);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,sourceLight);
	glLightfv(GL_LIGHT0,GL_POSITION,lightPos);
	glEnable(GL_LIGHT0);

//	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);

	// Enable color tracking
	glEnable(GL_COLOR_MATERIAL);
	
	// Set Material properties to follow glColor values
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

	// background
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f );    //(0.3f, 0.3f, 0.30f, 1.0f );

}

void SpecialKeys(int key, int x, int y)
{
	if(key == GLUT_KEY_UP)
		xRot-= 5.0f;

	if(key == GLUT_KEY_DOWN)
		xRot += 5.0f;

	if(key == GLUT_KEY_LEFT)
		zRot -= 5.0f;

	if(key == GLUT_KEY_RIGHT)
		zRot += 5.0f;

	if(key > 356.0f)
		xRot = 0.0f;

	if(key < -1.0f)
		xRot = 355.0f;

	if(key > 356.0f)
		zRot = 0.0f;

	if(key < -1.0f)
		zRot = 355.0f;

	// Refresh the Window
	glutPostRedisplay();
}

// Called to draw scene
void RenderScene()
{  

	// Clear the window with current clearing color
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Save matrix state and do the rotation
	glPushMatrix();
	glRotatef(xRot, 1.0f, 0.0f, 0.0f);
	glRotatef(zRot, 0.0f, 0.0f, 1.0f);
	   
 	glPolygonMode( GL_FRONT, GL_FILL );
	glPolygonMode( GL_BACK, GL_FILL );

    GLint trId;
    glNewList(1, GL_COMPILE);
 	for ( trId=0; trId<modelPtr->numTrs; trId++ )
	  {
       glBegin( GL_POLYGON );

       glColor3f(0.9, 0.9608, 0.9608);//(0.9608, 0.9608, 0.9608);
	      glNormal3fv( modelPtr->Normals[modelPtr->GetPointId(trId,0)] );
	      glVertex3f((GLfloat) modelPtr->Points[modelPtr->GetPointId(trId,0)][0],
			         (GLfloat) modelPtr->Points[modelPtr->GetPointId(trId,0)][1],
					 (GLfloat) modelPtr->Points[modelPtr->GetPointId(trId,0)][2]);
	      glNormal3fv( modelPtr->Normals[modelPtr->GetPointId(trId,1)]  );
 	      glVertex3f((GLfloat) modelPtr->Points[modelPtr->GetPointId(trId,1)][0],
			         (GLfloat) modelPtr->Points[modelPtr->GetPointId(trId,1)][1],
					 (GLfloat) modelPtr->Points[modelPtr->GetPointId(trId,1)][2]);
	      glNormal3fv( modelPtr->Normals[modelPtr->GetPointId(trId,2)] );
 	      glVertex3f((GLfloat) modelPtr->Points[modelPtr->GetPointId(trId,2)][0],
			         (GLfloat) modelPtr->Points[modelPtr->GetPointId(trId,2)][1],
					 (GLfloat) modelPtr->Points[modelPtr->GetPointId(trId,2)][2]);

        glEnd();
	  }
		
     glEndList();	
 
	glPointSize(2.0);
    glNewList(2, GL_COMPILE);
	ridgePtr->Head();
    GLint ptId;
    glBegin( GL_POINTS );
 	for ( ptId=0; ptId<ridgePtr->length(); ptId++ )
	  {
	   int curPt = ridgePtr->Retrieve();
          glColor3f(0.0, 0.0, 1.0);
	      glVertex3d((GLfloat) modelPtr->Points[curPt][0],
			  (GLfloat) modelPtr->Points[curPt][1],
			  (GLfloat) modelPtr->Points[curPt][2]);
		ridgePtr->Forward();
	  }
    glEnd();
    glEndList();	

   glNewList(3, GL_COMPILE);
   ridgeEdgesPtr->Head();
   glBegin( GL_LINES );
 	for ( ptId=0; ptId<ridgeEdgesPtr->length(); ptId++ )
	  {
	   int curPt = ridgeEdgesPtr->Retrieve();
           glColor3f(0.0, 0.0, 1.0);
	       glVertex3d((GLfloat) modelPtr->Points[curPt][0],
			  (GLfloat) modelPtr->Points[curPt][1],
			  (GLfloat) modelPtr->Points[curPt][2]);
		ridgeEdgesPtr->Forward();
	  }
    glEnd();
    glEndList();
float a =-0.1;
   glNewList(4, GL_COMPILE);
   glBegin( GL_LINES );
 	for ( ptId=0; ptId<modelPtr->numPts; ptId++ )
	  {
          glColor3f(0.0, 0.0, 1.0);
	       glVertex3d((GLfloat) modelPtr->Points[ptId][0],
			  (GLfloat) modelPtr->Points[ptId][1],
			  (GLfloat) modelPtr->Points[ptId][2]);
	  glVertex3d((GLfloat) modelPtr->Points[ptId][0]+a*modelPtr->Normals[ptId][0],
		(GLfloat) modelPtr->Points[ptId][1]+a*modelPtr->Normals[ptId][1],
		(GLfloat) modelPtr->Points[ptId][2]+a*modelPtr->Normals[ptId][2]);
	  }
    glEnd();
    glEndList();



   	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_POLYGON_OFFSET_FILL);
	glPolygonOffset(1.0, 1.0);                //to make edges look smooth 
    glCallList(1);
    glDisable(GL_POLYGON_OFFSET_FILL);

	glDisable(GL_LIGHTING);
	glDisable(GL_LIGHT0);
      glCallList(2);
      glCallList(3); 
//      glCallList(4);  	  
 	  glPopMatrix();

	  // Display the results
	glutSwapBuffers();
	
}


void ChangeSize( GLsizei w, GLsizei h)
{

   // Prevent a divide by zero
   if(h == 0)
		h = 1;
		
	// Set Viewport to window dimensions
    glViewport(0, 0, w, h);

	// Reset coordinate system
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	GLfloat nRange;
	nRange = (GLfloat) Abs(modelPtr->box->Xmin);
	if ( Abs(modelPtr->box->Xmax)>nRange ) nRange = (GLfloat) Abs(modelPtr->box->Xmax);
	if ( Abs(modelPtr->box->Ymax)>nRange ) nRange = (GLfloat) Abs(modelPtr->box->Ymax);
	if ( Abs(modelPtr->box->Ymin)>nRange ) nRange = (GLfloat) Abs(modelPtr->box->Ymin);
	if ( Abs(modelPtr->box->Zmax)>nRange ) nRange = (GLfloat) Abs(modelPtr->box->Zmax);
	if ( Abs(modelPtr->box->Zmin)>nRange ) nRange = (GLfloat) Abs(modelPtr->box->Zmin);

	nRange += 0.5*nRange;

	// Establish clipping volume (left, right, bottom, top, near, far)
    if (w <= h) 
		glOrtho (-nRange, nRange, -nRange*h/w, nRange*h/w, -nRange*2.0f, nRange*2.0f);
    else 
		glOrtho (-nRange*w/h, nRange*w/h, -nRange, nRange, -nRange*2.0f, nRange*2.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

}

