/*

   LList.cpp

   

   The class implementation for a singly linked list class

   LList, as well as Node, the node used for such lists.



   Throughout notice that pointers to new, existing, or

   temporary list nodes are all pointers to _N_, rather than

   some specific node type, such as Node.  This allows us

   to inherit from LList and specify (via the template) some

   other type for the list nodes (like DNode) and not have

   everything fall apart.

   

   Nic McPhee

   9/x/1995

*/



#ifndef _sLIST_CPP_

#define _sLIST_CPP_



#include <stdlib.h>

#include "sList.h"

#include "stdafx.h"

//----------------- Definition of LList member functions and friends





LList::LList()

// Default constructor.  Builds an empty list.

{   first = current = last = NULL; size = 0; }

      



LList::LList(const LList& l)

// copy constructor

{

   Node   *lptr = l.first,      // tracks through the argument list

            *nptr;         // tracks through this list

         

   if (lptr == NULL)      

   {   current = first = last = NULL; size = 0;}

   else

   {

      //---- Construct the first node in the list.

      nptr = new Node;

      nptr->value = lptr->value;

      if (lptr == l.current)

         current = nptr;

      first = nptr;

      lptr = lptr->next;

      

      //---- Track through the remainder of the list, copying nodes.

      while (lptr)

      {

         nptr->next = new Node;

         nptr = nptr->next;

         if (lptr == l.current)

            current = nptr;

         nptr->value = lptr->value;

         lptr = lptr->next;

      }

      nptr->next = NULL;

      last = nptr;

      size = l.size;

   }   

}



LList::~LList()

// Delete all nodes in the List.

{

   Clear();

}





void LList::Clear()

// Delete all nodes in the List.

{

   Node* nptr = first;

   current = first;

   while (current)

   {

      current = current->next;

      delete nptr;

      nptr = current;

   }

   first = current = last = NULL;

   size = 0;

}





LList& LList::operator= (const LList& l)

// assignment overload

{   

   if (this != &l)      // no need to assign to ourself

   {

      Clear();         // First, destroy the existing list.

   

      // Now build the new list

      Node   *lptr = l.first,      // tracks through the argument list

             *nptr;               // tracks through this list

         

      if (lptr == NULL)      

      {   current = first = last = NULL; size = 0;}

      else

      {

         // Construct the first node in the list.

         nptr = new Node;

         nptr->value = lptr->value;

         if (lptr == l.current)

            current = nptr;

         first = nptr;

         lptr = lptr->next;

      

         // Track through the remainder of the list, copying nodes.

         while (lptr)

         {

            nptr->next = new Node;

            nptr = nptr->next;

            if (lptr == l.current)

               current = nptr;

            nptr->value = lptr->value;

            lptr = lptr->next;

         }

         nptr->next = NULL;

         last = nptr;

         size = l.size;

      }

   }

   return *this;   

}

   



void LList::InsertBefore(int e)

// Insert e at the current position.

{

   Node* nptr = new Node;

   if (size == 0)                  // Inserting into an empty list.

   {

      nptr->value = e;

      nptr->next = NULL;

      current = first = last = nptr;

   }

   else                           // Inserting into a non-empy list.

   {

      nptr->value = current->value;

      nptr->next = current->next;

      current->value = e;

      current->next = nptr;

      if (current == last)

         last = nptr;

   }

   size++;

}

   



void LList::InsertAfter(int e)

// Insert e after the current position.

{

   Node* nptr = new Node;

   nptr->value = e;

   if (size == 0)      // Inserting into an empty List.

   {

      nptr->next = NULL;

      current = first = last = nptr;

   }

   else            // Inserting into a non-empty List.

   {

      nptr->next = current->next;

      current->next = nptr;

      if (current == last)

         last = nptr;

      current = nptr;

   }

   size++;

}

   

void LList::Remove()

// Delete the node at the current position.

{

   Node* nptr;

   if (size == 0)               

   // Warn the user if the list is already empty.

   {

//      Warn(size > 0, "Attempt to Remove an element from an empty List");

      return;

   }

   else if (size == 1)            

   // We're removing the only node in the list.

   {

      delete current;

      first = current = last = NULL;

   }   

   else if (current == last)   

   // We're removing the last node of a list of > 1 nodes.

   {

      nptr = first;

      while (nptr->next != current)

         nptr = nptr->next;

      nptr->next = NULL;

      delete last;

      current = last = nptr;

   }

   else                     

   // We're removing a node that's not the last one.

   {

      nptr = current->next;

      if (nptr == last)

         last = current;

      current->value = nptr->value;

      current->next = nptr->next;

      delete nptr;

   }

   size--;

}

   



void LList::Forward(void)

// Move current location to next node.

// Do nothing if current is at the tail of the list.

{

   if (current != last)

      current = current->next;

}

   



void LList::Back(void)

// Move current location to previous node.

// Do nothing if current is at the head of the list

{

   if (current != first)

   {

      Node* nptr = first;

      while (nptr->next != current)

         nptr = nptr->next;

      current = nptr;

   }

//   return l;

}

   



int LList::Retrieve() const   

// Return the element at the current position.

{ 

  

   return current->value;

}





 

void LList::Update(int e)

// Store x in current location.

{  

 //  Warn(size > 0, "Cannot Update an element in an empty list");



   if (size > 0)

      current->value = e;

}





int LList::Includes(int e)

// Returns 1 iff e is an element of the list.  If e is found, set current

// to the index of the first instance of e.

// NOTE: this assumes the existence of operator != on T.

{

   Node* ptr = first;

   while (ptr && (ptr->value != e))

      ptr = ptr->next;

   if (ptr != NULL)

      current = ptr;

   return (ptr != NULL);

}



#endif



