// softView.cpp : implementation of the CSoftView class
//

#include "stdafx.h"
#include "soft.h"

#include "softDoc.h"
#include "softView.h"
#include "MainFrm.h"
#include "renderer.h"
#include "myDlg.h"
#include "Diffusion.h"
#include "Normals.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSoftView

IMPLEMENT_DYNCREATE(CSoftView, CView)

BEGIN_MESSAGE_MAP(CSoftView, CView)
	//{{AFX_MSG_MAP(CSoftView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_ERASEBKGND()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_COMMAND(ID_VIEWOPTION_FLATSHADING, OnViewoptionFlatshading)
	ON_UPDATE_COMMAND_UI(ID_VIEW_STATUS_BAR, OnUpdateViewStatusBar)
	ON_COMMAND(ID_VIEWOPTION_SMOOTHSHADING, OnViewoptionSmoothshading)
	ON_UPDATE_COMMAND_UI(ID_VIEWOPTION_SMOOTHSHADING, OnUpdateViewoptionSmoothshading)
	ON_UPDATE_COMMAND_UI(ID_VIEWOPTION_FLATSHADING, OnUpdateViewoptionFlatshading)
	ON_COMMAND(ID_VIEWOPTION_WIREFRAME, OnViewoptionWireframe)
	ON_UPDATE_COMMAND_UI(ID_VIEWOPTION_WIREFRAME, OnUpdateViewoptionWireframe)
	ON_LBN_DBLCLK(IDC_MODELLIST, OnDblclkModellist)
	ON_COMMAND(ID_FEATURES_RIDGES, OnFeaturesRidges)
	ON_UPDATE_COMMAND_UI(ID_FEATURES_RIDGES, OnUpdateFeaturesRidges)
	ON_COMMAND(ID_FEATURES_RAVINES, OnFeaturesRavines)
	ON_UPDATE_COMMAND_UI(ID_FEATURES_RAVINES, OnUpdateFeaturesRavines)
	ON_COMMAND(ID_FEATURES_SNOWSURFACE, OnFeaturesSnowsurface)
	ON_UPDATE_COMMAND_UI(ID_FEATURES_SNOWSURFACE, OnUpdateFeaturesSnowsurface)
    ON_BN_CLICKED(IDC_SKETCH, OnDrawSketch)	
	ON_COMMAND(ID_CONTOUR, OnContour)
	ON_COMMAND(ID_SUBDIVIDE, OnSubdivide)
	ON_BN_CLICKED(IDC_APPLY, OnApply)
	ON_COMMAND(ID_RESET, OnReset)
	ON_COMMAND(ID_CHOOSE_BRUSH, OnChooseBrush)
	//}}AFX_MSG_MAP
    ON_MESSAGE(WM_USER_APPLY_BRUSH, OnViewApplyBrush)
    ON_MESSAGE(WM_USER_CLOSE, OnViewCloseBrushDlg)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSoftView construction/destruction

CSoftView::CSoftView()
{
	// TODO: add construction code here
  xRot = 0.0;
  yRot = 0.0;
  distance = 0.0;
  spinFactor = 0.5f;
  distanceFactor = 0.4f;

  m_drawMode = flat;
  m_ridge = off;
  m_ravine = off;
  m_both = 0;
  m_contour = off;
  m_surface = on;
  m_featureThr = 0;
  m_strokeWidth = 2.0;
  m_strokeLength = 0.01;

  m_hRC = NULL;
  m_pDC = NULL;
  m_brushDlg = NULL;

  m_pixelFormat = 0;
  movementMode = false;

}

CSoftView::~CSoftView()
{
}

BOOL CSoftView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

	return CView::PreCreateWindow(cs);
}

int CSoftView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here
	SetUpGL();
	m_drawMode = flat;
    BrushDlgShowing = FALSE;

	return 0;
}


void CSoftView::OnDestroy() 
{
	CView::OnDestroy();
	
	// TODO: Add your message handler code here
	wglMakeCurrent(NULL, NULL);
	wglDeleteContext(m_hRC);	

	if (m_pDC != NULL)
	{
		delete m_pDC;
		m_pDC = NULL;
	}	
}

/////////////////////////////////////////////////////////////////////////////
// CSoftView drawing

void CSoftView::OnDraw(CDC* pDC)
{
	CSoftDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here

   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if (pDoc->modelPtr != NULL )
	{ 
	  
	  glMatrixMode(GL_MODELVIEW);
      glPushMatrix();
	  glLoadIdentity(); 

      glRotatef(yRot, 0.0, 1.0, 0.0);
      glRotatef(xRot, 1.0, 0.0, 0.0);

	  switch(m_drawMode)
	  {
	    case sketch:
	     glEnable(GL_POLYGON_OFFSET_FILL);
	     glPolygonOffset(1.0, 1.0);             //to make edges look smooth  if there any
         drawWhiteModel(pDoc->modelPtr);
         glDisable(GL_POLYGON_OFFSET_FILL);
         glDisable(GL_LIGHTING);
	     glDisable(GL_LIGHT0);
         DrawSketch(pDoc); 
  	     glEnable(GL_LIGHTING);
	     glEnable(GL_LIGHT0);  break;
      
 		 case wire:
   	       if (m_surface == on)
		   {
	        glEnable(GL_POLYGON_OFFSET_FILL);
	        glPolygonOffset(1.0, 1.0);           //to make edges look smooth  if there any
            drawWhiteModel(pDoc->modelPtr);            
            glDisable(GL_POLYGON_OFFSET_FILL);
            glDisable(GL_LIGHTING);
	        glDisable(GL_LIGHT0);
	        glCallList(lstWireOnly);
  	        glEnable(GL_LIGHTING);
	        glEnable(GL_LIGHT0);
		    break;
		   }
  
		 case smooth:
   	       if (m_surface == on)
		   {
	        glEnable(GL_POLYGON_OFFSET_FILL);
	        glPolygonOffset(1.0, 1.0);             //to make edges look smooth  if there any
            glCallList(lstSmooth);
            glDisable(GL_POLYGON_OFFSET_FILL);
		    break;
		   }

         case flat:
		   if (m_surface == on)
		   {
	       glEnable(GL_POLYGON_OFFSET_FILL);
	       glPolygonOffset(1.0, 1.0);             //to make edges look smooth  if there any
           glCallList(lstFlat);
//		   this->drawModel(pDoc->modelPtr);
           glDisable(GL_POLYGON_OFFSET_FILL);
           break;
		   }
 
	  }		
	 
	  if ( (m_surface == off) && 
		       ((m_ridge == on)|| (m_ravine ==on) ||(m_contour == on)) )
		  {
	       glEnable(GL_POLYGON_OFFSET_FILL);
	       glPolygonOffset(1.0, 1.0);            //to make edges look smooth  if there any
           drawWhiteModel(pDoc->modelPtr);
           glDisable(GL_POLYGON_OFFSET_FILL);
		  }
      
	  if (m_ridge == on)
		{
         glDisable(GL_LIGHTING);
	     glDisable(GL_LIGHT0);
	     glCallList(lstRidge);
  	     glEnable(GL_LIGHTING);
	     glEnable(GL_LIGHT0);
		}
 	
	  if (m_ravine == on)
		{
         glDisable(GL_LIGHTING);
	     glDisable(GL_LIGHT0);
	     glCallList(lstRavine);
  	     glEnable(GL_LIGHTING);
	     glEnable(GL_LIGHT0);
		} 
      
	  if (m_contour == on)
	  {
        glDisable(GL_LIGHTING);
	    glDisable(GL_LIGHT0);
	    drawContour3D(pDoc->modelPtr);
  	    glEnable(GL_LIGHTING);
	    glEnable(GL_LIGHT0);
	  }

	 }
    glPopMatrix();
	SwapBuffers(wglGetCurrentDC());
/* 
	if (SwapBuffers(wglGetCurrentDC())==FALSE)
	{
			DWORD errorCode = GetLastError();
			CString errorStr;
			errorStr.Format("SwapBuffers returned error code %d.", errorCode);
			AfxMessageBox(errorStr);
			return;
	}
	*/
}

/////////////////////////////////////////////////////////////////////////////
// CSoftView diagnostics

#ifdef _DEBUG
void CSoftView::AssertValid() const
{
	CView::AssertValid();
}

void CSoftView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CSoftDoc* CSoftView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSoftDoc)));
	return (CSoftDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSoftView message handlers

BOOL CSoftView:: SetUpGL()
{

   m_pDC = new CClientDC(this);

   if (m_pDC == NULL) {
	 return FALSE;
   }

   if ( !SetupPixelFormat() ) {
	 return FALSE;
   }

   m_hRC = wglCreateContext(m_pDC->GetSafeHdc());
   if ( m_hRC == NULL ){
	 return FALSE;
   }

   if ( wglMakeCurrent(m_pDC->GetSafeHdc(), m_hRC)  == FALSE ){
	 return FALSE;
   }

	// white as a clear color
   glClearColor(1.0f, 1.0f, 1.0f, 0.0f);
   glClearDepth(1.0f);

   GLfloat ambient[] = {0.0, 0.0, 0.0, 1.0};
   GLfloat diffuse[] = {1.0, 1.0, 1.0, 1.0};
   GLfloat specular[] = {1.0, 1.0, 1.0, 1.0};
   GLfloat position[] = {0.0, 0.0, 30.0, 0.0};

   GLfloat lmodel_ambient[] = {0.2, 0.2, 0.2, 1.0};
   GLfloat local_view[] = {0.0};

   glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
   glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
   glLightfv(GL_LIGHT0, GL_POSITION, position);
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
   glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);

   GLfloat mat[4];
   mat[0] =(GLfloat) 0.2125; mat[1] = (GLfloat)0.1275; 
   mat[2] =(GLfloat) 0.054; mat[3] = (GLfloat)1.0;
   glMaterialfv(GL_FRONT, GL_AMBIENT, mat);
   mat[0] = 0.714; mat[1] = 0.4284; mat[2] = 0.18144;
   glMaterialfv(GL_FRONT, GL_DIFFUSE, mat);
   mat[0] = 0.393548; mat[1] = 0.271906; mat[2] = 0.166721;
   glMaterialfv(GL_FRONT, GL_SPECULAR, mat);
   GLfloat shine =0.2f;
   glMaterialf(GL_FRONT, GL_SHININESS, shine * 128.0);

   glFrontFace(GL_CCW);
   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
   glEnable(GL_NORMALIZE);
   glEnable(GL_DEPTH_TEST); 
 
   return TRUE;
}


BOOL CSoftView::SetupPixelFormat()
{
	// TODO: Add your specialized creation code here
     PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof(PIXELFORMATDESCRIPTOR),
		1,
		PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
		PFD_TYPE_RGBA,
		24,
		0,0,0,0,0,0,
		0,0,0,0,0,0,0,
		32,
		0,0,
		PFD_MAIN_PLANE,
		0,
		0,0,0
	};

	
	m_pixelFormat = ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd);
	if (m_pixelFormat == 0)
		return FALSE;

	if (FALSE == SetPixelFormat(m_pDC->GetSafeHdc(), m_pixelFormat, &pfd))
	 return FALSE;

	return TRUE;
}

void CSoftView::SetupViewVolume(polyData *model)
{

  RECT *rect = new RECT;
  GetClientRect(rect); 

  GLfloat w = (GLfloat)rect->right;
  GLfloat h = (GLfloat)rect->bottom;

  	// Set Viewport to window dimensions
  glViewport(0, 0, w, h);

  delete rect;

  GLfloat nRange = 0;

	if (model != NULL)
	 {
	   nRange = (GLfloat) fabs(model->box.Xmin);
	   if ( fabs(model->box.Xmax)>nRange ) 
		        nRange = (GLfloat) fabs(model->box.Xmax);
	   if ( fabs(model->box.Ymax)>nRange ) 
		        nRange = (GLfloat) fabs(model->box.Ymax);
	   if ( fabs(model->box.Ymin)>nRange ) 
		        nRange = (GLfloat) fabs(model->box.Ymin);
	   if ( fabs(model->box.Zmax)>nRange ) 
		        nRange = (GLfloat) fabs(model->box.Zmax);
	   if ( fabs(model->box.Zmin)>nRange ) 
		        nRange = (GLfloat) fabs(model->box.Zmin);

       nRange += (GLfloat)0.5*nRange;
	  }

	 glMatrixMode(GL_PROJECTION);
	 glLoadIdentity();

	  // Establish clipping volume (left, right, bottom, top, near, far)
      if (w <= h) 
		glOrtho(-nRange*zoom, nRange*zoom, -nRange*h/w*zoom, nRange*h/w*zoom, nRange, nRange*4.0);
      else 
		glOrtho(-nRange*w/h*zoom, nRange*w/h*zoom, -nRange*zoom, nRange*zoom, nRange, nRange*4.0 ); 

	  gluLookAt(0, 0, 2*nRange, 0, 0, 0, 0, 1, 0);

 	  glMatrixMode(GL_MODELVIEW);
      glLoadIdentity();

 }

void CSoftView::drawWhiteModel(polyData *model)
{

    float viewDir[4];
    this->getViewDir(viewDir);

    float viewDir3[3] = {viewDir[0], viewDir[1], viewDir[2]};

    GLint trId;
    glShadeModel(GL_FLAT); 
	glDisable(GL_LIGHTING);
	glColor3f(1.0f, 1.0f, 1.0f);
    glBegin( GL_TRIANGLES );
 	for ( trId=0; trId<model->numTrs; trId++ )
	  {
       
          glVertex3f((GLfloat) model->Points[model->GetPointId(trId,0)][0],
			         (GLfloat) model->Points[model->GetPointId(trId,0)][1],
					 (GLfloat) model->Points[model->GetPointId(trId,0)][2]);
 	      glVertex3f((GLfloat) model->Points[model->GetPointId(trId,1)][0],
			         (GLfloat) model->Points[model->GetPointId(trId,1)][1],
					 (GLfloat) model->Points[model->GetPointId(trId,1)][2]);
 	      glVertex3f((GLfloat) model->Points[model->GetPointId(trId,2)][0],
			         (GLfloat) model->Points[model->GetPointId(trId,2)][1],
					 (GLfloat) model->Points[model->GetPointId(trId,2)][2]);
        
	  }
      glEnd();
}

void CSoftView::drawModel(polyData *model)
{
  
   switch(m_drawMode)
   {
     case smooth   : drawSmooth(model); break;
	 case flat     : flatShading(model); break;
	 case wire     : drawWireframe(model); break; 
   }

}

void CSoftView::drawSmooth(polyData *model)
{ 
    GLint trId;

	Normals nmls = Normals(model->numPts);
	nmls.setInput(model);
	nmls.Proceed();

    glNewList(lstSmooth, GL_COMPILE);
    glShadeModel(GL_SMOOTH); 
    glBegin( GL_TRIANGLES );
 	for ( trId=0; trId<model->numTrs; trId++ )
	  {
		  glNormal3f((GLfloat) -nmls.getOutput()[model->GetPointId(trId,0)][0], 
			         (GLfloat) -nmls.getOutput()[model->GetPointId(trId,0)][1], 
			         (GLfloat) -nmls.getOutput()[model->GetPointId(trId,0)][2]); 		 

 	      glVertex3f((GLfloat) model->Points[model->GetPointId(trId,0)][0],
			         (GLfloat) model->Points[model->GetPointId(trId,0)][1],
					 (GLfloat) model->Points[model->GetPointId(trId,0)][2]);

		  glNormal3f((GLfloat) -nmls.getOutput()[model->GetPointId(trId,1)][0], 
			         (GLfloat) -nmls.getOutput()[model->GetPointId(trId,1)][1], 
			         (GLfloat) -nmls.getOutput()[model->GetPointId(trId,1)][2]);
 	      glVertex3f((GLfloat) model->Points[model->GetPointId(trId,1)][0],
			         (GLfloat) model->Points[model->GetPointId(trId,1)][1],
					 (GLfloat) model->Points[model->GetPointId(trId,1)][2]);
	      glNormal3f((GLfloat) -nmls.getOutput()[model->GetPointId(trId,2)][0],
			         (GLfloat) -nmls.getOutput()[model->GetPointId(trId,2)][1],
			         (GLfloat) -nmls.getOutput()[model->GetPointId(trId,2)][2]);
 	      glVertex3f((GLfloat) model->Points[model->GetPointId(trId,2)][0],
			         (GLfloat) model->Points[model->GetPointId(trId,2)][1],
					 (GLfloat) model->Points[model->GetPointId(trId,2)][2]);        
	  }
     glEnd();
     glEndList();

}

void CSoftView::drawWireframe(polyData *model)
{ 

    int trId;
	GLint ptId1, ptId2, ptId0;

    glNewList(lstWireOnly, GL_COMPILE);
	glShadeModel(GL_FLAT);
	glDisable(GL_LIGHTING);
	glColor3f(0.0f, 0.0f, 0.0f);

	glBegin( GL_LINES ); 
 	for ( trId=0; trId<model->numTrs; trId++ )
	  {
	    ptId0 = model->GetPointId(trId,0);

		ptId1 = model->GetPointId(trId,1);
		ptId2 = model->GetPointId(trId,2);      

          if (ptId0 <ptId1)
		  {
 	        glVertex3f((GLfloat) model->Points[ptId0][0],
			         (GLfloat) model->Points[ptId0][1],
					 (GLfloat) model->Points[ptId0][2]);
  	        glVertex3f((GLfloat) model->Points[ptId1][0],
			         (GLfloat) model->Points[ptId1][1],
					 (GLfloat) model->Points[ptId1][2]);
		  }


          if ( ptId1< ptId2 )
		  {
  	        glVertex3f((GLfloat) model->Points[ptId1][0],
			         (GLfloat) model->Points[ptId1][1],
					 (GLfloat) model->Points[ptId1][2]);
 	        glVertex3f((GLfloat) model->Points[ptId2][0],
			         (GLfloat) model->Points[ptId2][1],
					 (GLfloat) model->Points[ptId2][2]);
		  }


          if ( ptId2< ptId0 )
		  {

		    glVertex3f((GLfloat) model->Points[ptId2][0],
			          (GLfloat) model->Points[ptId2][1],
					  (GLfloat) model->Points[ptId2][2]);
 	        glVertex3f((GLfloat) model->Points[ptId0][0],
			          (GLfloat) model->Points[ptId0][1],
					  (GLfloat) model->Points[ptId0][2]);
		  }
        
	  }
     glEnd();
	 glEnable(GL_LIGHTING);
     glEndList();
}

void CSoftView::flatShading(polyData *model)
{ 
float *v = new float[3];
    GLint trId;
    glNewList(lstFlat, GL_COMPILE);
    glShadeModel(GL_FLAT); 
/*
glDisable(GL_LIGHTING);
glColor3f(1.0, 0, 0);
glBegin(GL_LINES);
glVertex3f(-5, 0, 0);
glVertex3f(5,0,0);
glColor3f(0.0, 1.0, 0);
glVertex3f(0, -5, 0);
glVertex3f(0,5,0);
glEnd();
glEnable(GL_LIGHTING);
*/
    glBegin( GL_TRIANGLES );
 	for ( trId=0; trId<model->numTrs; trId++ )
	  {  	
		  Geometry::trNormal(model->Points[model->GetPointId(trId,0)],
			        model->Points[model->GetPointId(trId,1)],
					model->Points[model->GetPointId(trId,2)], v);	
		  Geometry::normalize(v);
		  glNormal3f(-v[0],-v[1], -v[2]);

 	      glVertex3f((GLfloat) model->Points[model->GetPointId(trId,0)][0],
			         (GLfloat) model->Points[model->GetPointId(trId,0)][1],
					 (GLfloat) model->Points[model->GetPointId(trId,0)][2]);
	      
 	      glVertex3f((GLfloat) model->Points[model->GetPointId(trId,1)][0],
			         (GLfloat) model->Points[model->GetPointId(trId,1)][1],
					 (GLfloat) model->Points[model->GetPointId(trId,1)][2]);
	      
 	      glVertex3f((GLfloat) model->Points[model->GetPointId(trId,2)][0],
			         (GLfloat) model->Points[model->GetPointId(trId,2)][1],
					 (GLfloat) model->Points[model->GetPointId(trId,2)][2]);
        
	  }
     glEnd();
     glEndList();

	 delete []v;
}

void CSoftView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here
	CSoftDoc *doc = GetDocument();

	if (0>=cx || 0>=cy) 
		return;

	GLfloat w = (GLfloat)cx;
	GLfloat h = (GLfloat)cy;

	// Set Viewport to window dimensions
    glViewport(0, 0, w, h);

	  GLfloat nRange = 0;

	  if (doc->modelPtr != NULL)
	  {
	   nRange = (GLfloat) fabs(doc->modelPtr->box.Xmin);
	   if ( fabs(doc->modelPtr->box.Xmax)>nRange ) 
		        nRange = (GLfloat) fabs(doc->modelPtr->box.Xmax);
	   if ( fabs(doc->modelPtr->box.Ymax)>nRange ) 
		        nRange = (GLfloat) fabs(doc->modelPtr->box.Ymax);
	   if ( fabs(doc->modelPtr->box.Ymin)>nRange ) 
		        nRange = (GLfloat) fabs(doc->modelPtr->box.Ymin);
	   if ( fabs(doc->modelPtr->box.Zmax)>nRange ) 
		        nRange = (GLfloat) fabs(doc->modelPtr->box.Zmax);
	   if ( fabs(doc->modelPtr->box.Zmin)>nRange ) 
		        nRange = (GLfloat) fabs(doc->modelPtr->box.Zmin);

       nRange += (GLfloat)0.5*nRange;
	  }

	  // Reset coordinate system
	  glMatrixMode(GL_PROJECTION);
	  glLoadIdentity();

	  // Establish clipping volume (left, right, bottom, top, near, far)
      if (w <= h) 
		glOrtho (-nRange*zoom, nRange*zoom, -nRange*h/w*zoom, nRange*h/w*zoom, nRange, nRange*4.0);
      else 
		glOrtho (-nRange*w/h*zoom, nRange*w/h*zoom, -nRange*zoom, nRange*zoom, nRange, nRange*4.0 ); 

	  gluLookAt(0, 0, 2*nRange, 0, 0, 0, 0, 1, 0);
	
   	glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
	
}

BOOL CSoftView::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default
	
//	return CView::OnEraseBkgnd(pDC);
	return TRUE;          //tell Windows not to erase the bckgrd
}

void CSoftView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default

	CPoint delta = point;
	delta.x -= mouseStartPos.x;
	delta.y -= mouseStartPos.y;

	if (movementMode == true ){
		xRot += spinFactor * delta.y;
		yRot += spinFactor * delta.x;
        Invalidate();
        mouseStartPos = point; 
	}
	
	CView::OnMouseMove(nFlags, point);
}

void CSoftView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	movementMode = true;
    mouseStartPos = point;
	
	CView::OnLButtonDown(nFlags, point);
}

void CSoftView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default

	movementMode = false;
		
	CView::OnLButtonUp(nFlags, point);
}

void CSoftView::OnViewoptionFlatshading() 
{
	// TODO: Add your command handler code here
	CSoftDoc *doc = GetDocument();

	m_drawMode = flat;
    this->flatShading(doc->modelPtr);
	Invalidate();	
}

void CSoftView::OnUpdateViewStatusBar(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if ( m_drawMode == flat )
		pCmdUI->SetCheck(TRUE);
	else
        pCmdUI->SetCheck(FALSE);
}

void CSoftView::OnViewoptionSmoothshading() 
{
	// TODO: Add your command handler code here

	CSoftDoc *doc = GetDocument();

	m_drawMode = smooth;
    this->drawSmooth(doc->modelPtr);
	Invalidate();		
}

void CSoftView::OnUpdateViewoptionSmoothshading(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if ( m_drawMode == smooth )
		pCmdUI->SetCheck(TRUE);
	else
        pCmdUI->SetCheck(FALSE);	
}

void CSoftView::OnUpdateViewoptionFlatshading(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if ( m_drawMode == flat )
		pCmdUI->SetCheck(TRUE);
	else
        pCmdUI->SetCheck(FALSE);	
}

void CSoftView::OnViewoptionWireframe() 
{
	// TODO: Add your command handler code here
	CSoftDoc *doc = GetDocument();

	m_drawMode = wire;
    this->drawWireframe(doc->modelPtr);
	Invalidate();
}

void CSoftView::OnUpdateViewoptionWireframe(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if ( m_drawMode == wire )
		pCmdUI->SetCheck(TRUE);
	else
        pCmdUI->SetCheck(FALSE);
	
}

void CSoftView::OnDblclkModellist() 
{
	// TODO: Add your control notification handler code here
  CString path, type;
  
  CMainFrame *frame = (CMainFrame*) GetParent();
  CListBox *box = (CListBox*) frame->m_wndDlgBar.GetDlgItem(IDC_MODELLIST); 

  int nIndex = box->GetCurSel();
  if (nIndex == LB_ERR )
	  return;
  CString name;
 switch (nIndex)
 {
   case 0: path = "C:\\Lena\\3Data\\vtk9\\bunny.vtk"; type = "vtk"; name = "Bunny"; break;
   case 1: path = "C:\\Lena\\3Data\\vtk9\\cowMan.vtk"; type = "vtk"; name = "Cow"; break;
   case 2: path = "C:\\Lena\\3Data\\vtk9\\goodSphere.vtk"; type = "vtk"; name = "Sphere";break;
   case 3: path = "C:\\Lena\\3Data\\ply\\venus.ply2"; type = "ply2"; name ="Venus"; break;
   case 4: path = "C:\\Lena\\3Data\\ply\\octa.ply2"; type = "ply2"; name = "octa"; break;
   case 5: path = "C:\\Lena\\3Data\\vtk9\\2torus_irr.vtk";type="vtk"; name="2 Torus";break;
   case 6: path = "C:\\Lena\\3Data\\ply\\jaw.ply2"; type = "ply2"; name = "Jaw";break;
   case 7: path = "C:\\Lena\\3Data\\vtk9\\teapot.vtk"; type = "vtk";name = "Teapot"; break;
 }

  CSoftDoc *doc = GetDocument();
 
  if (doc->modelPtr != NULL )  doc->ClearDoc();

  doc->fileMngr = new fileManager();
  doc->fileMngr->setFile(path, type);
  doc->modelPtr = new polyData();
  if (!doc->fileMngr->readFileToModel(path, doc->modelPtr))
	  return;

  CString s = path;
  doc->m_modelInfo.Format("  Model: "+ name  +
		                  "     Points: %d     Faces: %d", 
		                  doc->modelPtr->numPts, doc->modelPtr->numTrs);


  doc->modelPtr->InitModel();
  InitView();
  SetupViewVolume(doc->modelPtr);
 
 
  flatShading(doc->modelPtr);
  
  Invalidate();
}



void CSoftView::drawRidge(polyData *modelPtr, LList &ridge, float m_featureThr)
{
 	CSoftDoc* pDoc = GetDocument();

	glPointSize(2.0);
    glNewList(lstRidge, GL_COMPILE);
	ridge.Head();
    GLint ptId;
    glBegin( GL_POINTS );
    glColor3f(1.0, 1.0, 0.0);
 	for ( ptId=0; ptId<ridge.length(); ptId++ )
	  {
	   int curPt = ridge.Retrieve();
       if ( pDoc->modelCurv->Kmax[curPt] < m_featureThr)
	   {
         ridge.Forward();
		 continue;
	   }    
	   

	      glVertex3d((GLfloat) modelPtr->Points[curPt][0],
			  (GLfloat) modelPtr->Points[curPt][1],
			  (GLfloat) modelPtr->Points[curPt][2]);
		ridge.Forward();
	  }
    glEnd();
    glEndList();	

}



void CSoftView::OnFeaturesRidges() 
{
	// TODO: Add your command handler code here
  if (m_ridge == on)
  {
   m_ridge = off;
  }
  else
  {
   m_ridge = on;

 
	// TODO: Add your command handler code here
   CSoftDoc *doc = GetDocument();

   if (doc->feature == NULL)
   {
    doc->feature = new featureHandler();
   }

   if (doc->feature->ridge.length() == 0)
   {

    if (doc->modelCurv == NULL)
    {
	 doc->modelCurv = new curvature(doc->modelPtr->numPts);
     doc->modelCurv->setModel(doc->modelPtr);
     doc->modelCurv->getPrincCurv();
    }

    doc->feature->getRidge(doc->modelPtr, doc->modelCurv, 0);
  
   }

  this->drawRidge(doc->modelPtr, doc->feature->ridge, m_featureThr);
  }

  Invalidate();	
  	
}

void CSoftView::OnUpdateFeaturesRidges(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if ( m_ridge == on )
		pCmdUI->SetCheck(TRUE);
	else
        pCmdUI->SetCheck(FALSE);	
}

void CSoftView::drawRavine(polyData *modelPtr, LList &ravine, float m_featureThr)
{
	CSoftDoc* pDoc = GetDocument();

	glPointSize(2.0);
    glNewList(lstRavine, GL_COMPILE);
	ravine.Head();
    GLint ptId;
    glBegin( GL_POINTS );
    glColor3f(0.0, 0.0, 0.0);
 	for ( ptId=0; ptId<ravine.length(); ptId++ )
	  {
	   int curPt = ravine.Retrieve();
       if ( pDoc->modelCurv->Kmin[curPt] > - m_featureThr)
	   {
         ravine.Forward();
		 continue;
	   }    
	   
	      glVertex3d((GLfloat) modelPtr->Points[curPt][0],
			  (GLfloat) modelPtr->Points[curPt][1],
			  (GLfloat) modelPtr->Points[curPt][2]);
		ravine.Forward();
	   
	  }
    glEnd();
    glEndList();
	
}

void CSoftView::OnFeaturesRavines() 
{
	// TODO: Add your command handler code here
	// TODO: Add your command handler code here
  if (m_ravine == on)
  {
   m_ravine = off;
  }
  else
  {

   m_ravine = on;
 
	// TODO: Add your command handler code here
   CSoftDoc *doc = GetDocument();

   if (doc->feature == NULL)
   {
     doc->feature = new featureHandler();
   }
 
   if (doc->feature->ravine.length() == 0)
   {
    if (doc->modelCurv == NULL)
    {
	 doc->modelCurv = new curvature(doc->modelPtr->numPts);
     doc->modelCurv->setModel(doc->modelPtr);
     doc->modelCurv->getPrincCurv();
    }

    doc->feature->getRavine(doc->modelPtr, doc->modelCurv, 0);    
   }

   this->drawRavine(doc->modelPtr, doc->feature->ravine, m_featureThr);
  }

  Invalidate();	
  		
}

void CSoftView::OnUpdateFeaturesRavines(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if ( m_ravine == on )
		pCmdUI->SetCheck(TRUE);
	else
        pCmdUI->SetCheck(FALSE);	
}

void CSoftView::OnFeaturesSnowsurface() 
{
	// TODO: Add your command handler code here
	// TODO: Add your command handler code here

	if (m_surface == on) m_surface =  off;
	else m_surface =  on;	
	Invalidate();	
}

void CSoftView::OnUpdateFeaturesSnowsurface(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
	if ( m_surface == on )
		pCmdUI->SetCheck(TRUE);
	else
        pCmdUI->SetCheck(FALSE);	
}

void CSoftView::OnDrawSketch() 
{
   m_drawMode = sketch;
   m_surface = off;
   m_ridge = off;
   m_ravine = off;

   Invalidate();
} 

void CSoftView::DrawSketch(CSoftDoc *doc) 
{

 	// TODO: Add your control notification handler code here
  
   if (doc->modelCurv == NULL)
    {
	 doc->modelCurv = new curvature(doc->modelPtr->numPts);
     doc->modelCurv->setModel(doc->modelPtr);
     doc->modelCurv->getPrincCurv();
    }

    if (doc->feature == NULL)
	{
	  doc->feature = new featureHandler();
	}

    if (doc->feature->ravine.length() == 0)
    {
     doc->feature->getRavine(doc->modelPtr, doc->modelCurv, 0);

	}

    if (doc->feature->ridge.length() == 0)
    {
     doc->feature->getRidge(doc->modelPtr, doc->modelCurv, 0);

	}


	if ( doc->modelCurv->ifSmoothedTmax == FALSE)
	{
       doc->modelCurv->smoothField(doc->modelCurv->Tmax, 
		                            doc->modelCurv->SmoothedTmax, doc->modelPtr, 3);
       doc->modelCurv->ifSmoothedTmax = TRUE;
	} 
	
	if ( doc->modelCurv->ifSmoothedTmin == FALSE)
	{
       doc->modelCurv->smoothField(doc->modelCurv->Tmin, 
		                            doc->modelCurv->SmoothedTmin, doc->modelPtr, 3);
       doc->modelCurv->ifSmoothedTmin = TRUE;
	}  
	
   float viewDir[4];
   this->getViewDir(viewDir);

   float viewDir3[3] = {viewDir[0], viewDir[1], viewDir[2]};

   sketchRavine(doc->modelCurv->SmoothedTmax, doc, viewDir3,m_strokeLength, 
		                                           m_strokeWidth, m_featureThr);   

 //  sketchRidge(doc->modelCurv->SmoothedTmin, doc->modelPtr, viewDir3,
//		                 doc->feature->ridge, doc->modelCurv, m_strokeLength, 
//		                                           m_strokeWidth, m_featureThr);   


   LList *contour = new LList();
   getContour3D(doc->modelPtr, viewDir, contour);	
  
//   sketchContour(contour, viewDir3, doc->modelPtr, m_strokeLength, m_strokeWidth);

   CSoftView::drawContour3D(doc->modelPtr);
   
   contour->Clear();
   delete contour;

}

void CSoftView::OnContour() 
{
	// TODO: Add your command handler code here
  m_contour = on;
  Invalidate();
}

void CSoftView::drawContour3D(polyData *modelPtr) 
{

  float viewDir[4];

  this->getViewDir(viewDir);

  LList *contour = new LList();
  getContour3D(modelPtr, viewDir, contour);	

  glPointSize(1.2*m_strokeWidth);
 	contour->Head();
    GLint ptId;
    glBegin( GL_LINES );
    glColor3f(0.0, 0.0, 0.0);
 	for ( ptId=0; ptId < contour->length()/2; ptId++ )
	  {
	   int curPt = contour->Retrieve();
       contour->Forward();
	   int curPt_next = contour->Retrieve();
       contour->Forward();    
	      glVertex3d((GLfloat) modelPtr->Points[curPt][0],
			  (GLfloat) modelPtr->Points[curPt][1],
			  (GLfloat) modelPtr->Points[curPt][2]);
	      glVertex3d((GLfloat) modelPtr->Points[curPt_next][0],
			  (GLfloat) modelPtr->Points[curPt_next][1],
			  (GLfloat) modelPtr->Points[curPt_next][2]);
		
	  }
    glEnd();
    glFlush();

	contour->Clear();
	delete contour;
}

void CSoftView::OnSubdivide() 
{
	// TODO: Add your command handler code here
	subdivider *sub = new subdivider();

	CSoftDoc *doc = GetDocument();

    sub->setInputModel(doc->modelPtr);
    sub->PlanarSBD();

	if (doc->modelCurv != NULL) 
	{
		delete doc->modelCurv;
		doc->modelCurv = NULL;
	}

	if (doc->feature != NULL)
	{
      delete doc->feature;
	  doc->feature = NULL;
	}

    if (doc->modelPtr != NULL)
	{
      delete doc->modelPtr;
 	} 

	doc->modelPtr = new polyData();

	doc->modelPtr->copyPointsFrom(sub->getOutput());
    doc->modelPtr->copyTrianglesFrom(sub->getOutput());

	doc->modelPtr->InitModel();

	delete sub;

	CSoftView::drawModel(doc->modelPtr);
    Invalidate();
}

void CSoftView::OnApply() 
{
	// TODO: Add your control notification handler code here

    CMainFrame *frame = (CMainFrame*) GetParent(); 

    CString sTr, sW, sL;
	CEdit *EditTr = (CEdit *) frame->m_wndDlgBar.GetDlgItem(IDC_EDIT_THR);
	CEdit *EditW = (CEdit *) frame->m_wndDlgBar.GetDlgItem(IDC_EDIT_WIDTH);
	CEdit *EditL = (CEdit *) frame->m_wndDlgBar.GetDlgItem(IDC_EDIT_LENGTH);
	
	EditTr->GetWindowText(sTr);
    EditW->GetWindowText(sW);
    EditL->GetWindowText(sL);
   
	if ( m_featureThr != atof(sTr))
	{
      m_featureThr = atof(sTr);

      CSoftDoc *doc = GetDocument();
	  if(doc->modelPtr != NULL)
	  {
	    if (m_ridge == on )
		 {
		   CSoftView::drawRidge(doc->modelPtr, doc->feature->ridge, m_featureThr);	
		 }

	     if (m_ravine == on)
		 {
			CSoftView::drawRavine(doc->modelPtr, doc->feature->ravine, m_featureThr);		   
         }
	   }
	  
	}
         m_strokeWidth = atof(sW);
         m_strokeLength = atof(sL);
	
		 Invalidate();
    
}


void CSoftView::getViewDir(float *newView)
{
  float v[4] = {0, 0, -1, 1};
  float M[16];
  glGetFloatv(GL_MODELVIEW_MATRIX, M);

  float M_trans[16] = {  M[0], M[4], M[8], M[11], 
	                     M[1], M[5], M[9], M[13],
						 M[2], M[6], M[10], M[14],
						 M[3], M[7], M[11], M[15]};

  Geometry::MatrByVect(M_trans, v, newView);

}

void CSoftView::OnReset() 
{
	// TODO: Add your command handler code here
	m_drawMode = flat;
	m_surface = on;
	m_ridge = off;
	m_ravine = off;
	m_contour = off;
	xRot =0.0f;
	yRot =0.0f;

	Invalidate();
}


void CSoftView::OnChooseBrush() 
{
	// TODO: Add your command handler code here
	if (!BrushDlgShowing)
	{
	  BrushDlgShowing = TRUE;
	  m_brushDlg = new CBrushOptionsDlg;
      m_brushDlg->Create(IDD_BRUSH_DIALOG, this);
      m_brushDlg->ShowWindow(SW_RESTORE);
	}
}

LRESULT CSoftView::OnViewApplyBrush(WPARAM wPar, LPARAM lPar)
{
   Invalidate();
   return 0;
}

LRESULT CSoftView::OnViewCloseBrushDlg(WPARAM wPar, LPARAM lPar)
{
  BrushDlgShowing = FALSE;
  return 0;
}

/*
void CSoftView::OnDiffusion() 
{
	// TODO: Add your command handler code here
    CSoftDoc *doc = GetDocument();
	if (doc->modelPtr == NULL ) return;
	 
	Diffusion* diff = new Diffusion(doc->modelPtr->numPts);
	diff->SetInput(doc->modelPtr);
	diff->Proceed();

	if (doc->modelCurv != NULL) 
	{
		delete doc->modelCurv;
		doc->modelCurv = NULL;
	}

	if (doc->feature != NULL)
	{
      delete doc->feature;
	  doc->feature = NULL;
	}
 

   doc->modelPtr->Translate2Origin();
   doc->modelPtr->makeBoundBox();

   SetupViewVolume(doc->modelPtr);
   drawModel(doc->modelPtr);
   Invalidate();
   
   delete diff;	
}
*/
void CSoftView::InitView()
{
  m_drawMode = flat;
  m_ridge = off;
  m_ravine = off;
  m_contour = off;
  m_surface = on;
  m_featureThr = 0;
  m_strokeWidth = 2.0;
  m_strokeLength = 0.01;
  zoom = 1.0f;
  
  xRot = 0.0;
  yRot = 0.0;
  distance = 0.0;
  spinFactor = 0.3f;
  distanceFactor = 0.4f;
}
