// subdivider.cpp: implementation of the subdivider class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "soft.h"
#include "subdivider.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

subdivider::subdivider()
{
  input = new polyData();
  output = new polyData();
}

subdivider::~subdivider()
{
   delete input;
   delete output;
}

void subdivider::setInputModel(polyData *dataIn)
{
  this->input->copyPointsFrom(dataIn);
  this->input->copyTrianglesFrom(dataIn);

  this->input->BuildLinks();
}


void subdivider::PlanarSBD()
{

  int temp;
  LList *inc = new LList();

  int nPts = this->input->numPts;
  int nTrs = this->input->numTrs;


  //find a number of edges in the model. 
  //The edge number is equal to the new points  number
  int numEdges =0 ;
  for (int i=0; i<nPts; i++)
  {
    temp = this->input->incidentVts(i, inc);
	numEdges += temp;
	inc->Clear();
  }
  
  numEdges /= 2;
  
  delete inc;

  float midle[3];
  LList **m;
  m = new LList* [nPts];   //
  for (i=0; i<nPts; i++ )
	{
	  m[i] = new LList;
	}

  this->output->SetNumPoints(nPts+numEdges);
  this->output->allocatePoints(nPts+numEdges);

  for (int ptId=0; ptId<nPts; ptId++)
  {
    this->output->setPoint(ptId, this->input->Points[ptId]);
  }

  int newCount = nPts; 
  for (int trId =0; trId<nTrs; trId++)
  {   
	 for (int k=0; k<3; k++)
	 {
	  int p1 = this->input->GetPointId(trId, k);
      int p2 = this->input->GetPointId(trId, (k+1)%3);

	  if (p1 > p2)
	  { 
		if (!this->input->isBoundaryEdge(p1, p2) ) continue;
		else 
		{
         temp = p1;
	     p1 = p2;
	     p2 = temp;
		}
	  }

	  midle[0] = 0.5f*(this->input->Points[p1][0]+this->input->Points[p2][0]);
	  midle[1] = 0.5f*(this->input->Points[p1][1]+this->input->Points[p2][1]);
	  midle[2] = 0.5f*(this->input->Points[p1][2]+this->input->Points[p2][2]);

      m[p1]->InsertAfter(p2);
	  m[p1]->InsertAfter(newCount); 

	  this->output->setPoint(newCount, midle);
	  
	  newCount++;
	 }
  }

    this->output->SetNumTrs(4*nTrs); 
    this->output->allocatePolys(4*nTrs);

    int newTrId =0;
	int p0, p1, p2, p01, p12, p20;
    for (trId=0; trId<nTrs; trId++)
	{

      p0 = this->input->GetPointId(trId, 0);
      p1 = this->input->GetPointId(trId, 1);

      if (p0>p1)
	  {
       temp = p0;
	   p0 = p1;
	   p1 = temp;
	  }

      m[p0]->Head();	  
	  while( m[p0]->Retrieve() != p1)
	  {
	    m[p0]->Forward();
        m[p0]->Forward();
	  }
      m[p0]->Forward();
	  p01 = m[p0]->Retrieve();


      p1 = this->input->GetPointId(trId, 1);
      p2 = this->input->GetPointId(trId, 2);
      if (p1>p2)
	  {
       temp = p1;
	   p1 = p2;
	   p2 = temp;
	  }

      m[p1]->Head();	  
	  while( m[p1]->Retrieve() != p2)
	  {
	    m[p1]->Forward();
        m[p1]->Forward();
	  }
      m[p1]->Forward();
	  p12 = m[p1]->Retrieve();

 
	  p0 = this->input->GetPointId(trId, 0);
      p2 = this->input->GetPointId(trId, 2);
      if (p2>p0)
	  {
       temp = p2;
	   p2 = p0;
	   p0 = temp;
	  }

      m[p2]->Head();	  
	  while( m[p2]->Retrieve() != p0)
	  {
	    m[p2]->Forward();
        m[p2]->Forward();
	  }
      m[p2]->Forward();
	  p20 = m[p2]->Retrieve();


      p0 = this->input->GetPointId(trId, 0);
      p1 = this->input->GetPointId(trId, 1);
      p2 = this->input->GetPointId(trId, 2);

      this->output->Triangles[newTrId][0] = p0;
      this->output->Triangles[newTrId][1] = p01;
      this->output->Triangles[newTrId][2] = p20;

	  newTrId++;
 
	  this->output->Triangles[newTrId][0] = p1;
      this->output->Triangles[newTrId][1] = p01;
      this->output->Triangles[newTrId][2] = p12;

	  newTrId++;

	  this->output->Triangles[newTrId][0] = p2;
      this->output->Triangles[newTrId][1] = p12;
      this->output->Triangles[newTrId][2] = p20;
 
	  newTrId++;

	  this->output->Triangles[newTrId][0] = p01;
      this->output->Triangles[newTrId][1] = p12;
      this->output->Triangles[newTrId][2] = p20;

	  newTrId++;
	}  

	
   for (i=0; i<nPts; i++ )
	{
	  delete m[i];
	}

   
    delete m;
}

polyData* subdivider::subdivAlongFeature(LList feature, polyData *model)
{
/*
  int temp, i;
  LList *inc = new LList();

  int nPts =feature.length(); 

  //find a number of edges in the feature 1-neighbouhod. 
  //The edge number is equal to the new points  number
  int numEdges =0 ;
  feature.Head();
  for (i=0; i<nPts; i++)
  {
	int ptId = feature.Retrieve();
    temp = model->incidentVts(ptId, inc);
	numEdges += temp;
	inc->Clear();
	feature.Forward();
  }

  delete inc;

  numEdges /= 2;    

// find number of triangles in neibhd
//
 int *nei = new int[5];
  
 for (i=0; i<nPts; i++)
  {
	int ptId = feature.Retrieve();
 
	for (int cur=0; cur<model->Neis[ptId].numNei; cur++)
	{
      int trId = model->Neis[ptId].pointNei[cur];
      
	  int p0 = model->GetPointId(trId, 0);
	  int p1 = model->GetPointId(trId, 1);
	  int p2 = model->GetPointId(trId, 2);

	  if (p0<p1)
	  {
        int t = model->GetEdgeNei(p0, p1, nei);
		neibhd->InsertAfter(nei[0]);
		neibhd->Forward();
		neibhd->InsertAfter(nei[1]);
	  }

	  if (p1<p2)
	  {
        int t = model->GetEdgeNei(p0, p1, nei);
		neibhd->InsertAfter(nei[0]);
		neibhd->Forward();
		neibhd->InsertAfter(nei[1]);
	  }

	  if (p2<p0)
	  {
        int t = model->GetEdgeNei(p0, p1, nei);
		neibhd->InsertAfter(nei[0]);
		neibhd->Forward();
		neibhd->InsertAfter(nei[1]);
	  }
	}          //for 1-circle

    feature.Forward();
  }

  delete [] nei;

  polyData data = polyData(nPts+numEdges, 0);


/*
  int temp;
  LList *inc = new LList();

  int nPts =feature.length(); 

  //find a number of edges in the feature 1-neighbouhod. 
  //The edge number is equal to the new points  number
  int numEdges =0 ;
  for (int i=0; i<nPts; i++)
  {
    temp = this->input->incidentVts(i, inc);
	numEdges += temp;
	inc->Clear();
  }

  delete inc;

  numEdges /= 2;    

// find number of triangles in neibhd
//


  polyData data = polyData(nPts+numEdges, 0);



  float midle[3];
  LList **m;
  m = new LList* [nPts];   //
  for (i=0; i<nPts; i++ )
	{
	  m[i] = new LList;
	}

  this->output->SetNumPoints(nPts+numEdges);
  this->output->allocatePoints(nPts+numEdges);

  for (int ptId=0; ptId<nPts; ptId++)
  {
    this->output->setPoint(ptId, this->input->Points[ptId]);
  }

  int newCount = nPts; 
  for (int trId =0; trId<nTrs; trId++)
  {   
	 for (int k=0; k<3; k++)
	 {
	  int p1 = this->input->GetPointId(trId, k);
      int p2 = this->input->GetPointId(trId, (k+1)%3);

	  if (p1 > p2)
	  { 
		if (!this->input->isBoundaryEdge(p1, p2) ) continue;
		else 
		{
         temp = p1;
	     p1 = p2;
	     p2 = temp;
		}
	  }

	  midle[0] = 0.5f*(this->input->Points[p1][0]+this->input->Points[p2][0]);
	  midle[1] = 0.5f*(this->input->Points[p1][1]+this->input->Points[p2][1]);
	  midle[2] = 0.5f*(this->input->Points[p1][2]+this->input->Points[p2][2]);

      m[p1]->InsertAfter(p2);
	  m[p1]->InsertAfter(newCount); 

	  this->output->setPoint(newCount, midle);
	  
	  newCount++;
	 }
  }

    this->output->SetNumTrs(4*nTrs); 
    this->output->allocatePolys(4*nTrs);

    int newTrId =0;
	int p0, p1, p2, p01, p12, p20;
    for (trId=0; trId<nTrs; trId++)
	{

      p0 = this->input->GetPointId(trId, 0);
      p1 = this->input->GetPointId(trId, 1);

      if (p0>p1)
	  {
       temp = p0;
	   p0 = p1;
	   p1 = temp;
	  }

      m[p0]->Head();	  
	  while( m[p0]->Retrieve() != p1)
	  {
	    m[p0]->Forward();
        m[p0]->Forward();
	  }
      m[p0]->Forward();
	  p01 = m[p0]->Retrieve();


      p1 = this->input->GetPointId(trId, 1);
      p2 = this->input->GetPointId(trId, 2);
      if (p1>p2)
	  {
       temp = p1;
	   p1 = p2;
	   p2 = temp;
	  }

      m[p1]->Head();	  
	  while( m[p1]->Retrieve() != p2)
	  {
	    m[p1]->Forward();
        m[p1]->Forward();
	  }
      m[p1]->Forward();
	  p12 = m[p1]->Retrieve();

 
	  p0 = this->input->GetPointId(trId, 0);
      p2 = this->input->GetPointId(trId, 2);
      if (p2>p0)
	  {
       temp = p2;
	   p2 = p0;
	   p0 = temp;
	  }

      m[p2]->Head();	  
	  while( m[p2]->Retrieve() != p0)
	  {
	    m[p2]->Forward();
        m[p2]->Forward();
	  }
      m[p2]->Forward();
	  p20 = m[p2]->Retrieve();


      p0 = this->input->GetPointId(trId, 0);
      p1 = this->input->GetPointId(trId, 1);
      p2 = this->input->GetPointId(trId, 2);

      this->output->Triangles[newTrId][0] = p0;
      this->output->Triangles[newTrId][1] = p01;
      this->output->Triangles[newTrId][2] = p20;

	  newTrId++;
 
	  this->output->Triangles[newTrId][0] = p1;
      this->output->Triangles[newTrId][1] = p01;
      this->output->Triangles[newTrId][2] = p12;

	  newTrId++;

	  this->output->Triangles[newTrId][0] = p2;
      this->output->Triangles[newTrId][1] = p12;
      this->output->Triangles[newTrId][2] = p20;
 
	  newTrId++;

	  this->output->Triangles[newTrId][0] = p01;
      this->output->Triangles[newTrId][1] = p12;
      this->output->Triangles[newTrId][2] = p20;

	  newTrId++;
	}  

	
   for (i=0; i<nPts; i++ )
	{
	  delete m[i];
	}

   
    delete m;
*/
       return  model;
}


polyData* subdivider::getOutput()
{
   return(this->output);
}